# include <iostream>
# include <iomanip>
# include "Glyph.h"  

# include "pgm2asc.h"
# include "gocr.h"    // Box

// ******************************************************************
// CONSTRUCTORS

# include <iostream>
using namespace std;

namespace SomeName {

  Glyph::Glyph(Element* parent, void* data) : Element(parent, Coord(0,0), Coord(0,0)) {
    // NOTE: The Coord(0,0) in the init list is rather dirty, but we
    // do promptly initialize 'topleft' and 'size' within this
    // constructor as long as 'data' is provided!
    Box* box;
    if ( data ) {
      // Horrible type-violation - temporary.
      box = (Box*)data;
      this->topleftRef().x = box->x0;
      this->topleftRef().y = box->y0;
      this->bottomrightRef().x = box->x1;
      this->bottomrightRef().y = box->y1;
    } else {
      cerr << "ERROR: Creation of Glyph without underlying Box is currently not safe!" << endl;
      box = (Box*) malloc_box(NULL);
    }
    this->boxIs(box);
  }

  // Code copied from 'malloc_box'
  Glyph::Glyph(Glyph* glyph) : Element(glyph->parentRef(), Coord(0,0), Coord(0,0)) {
    Box* box = (Box*) malloc_box(glyph->boxRef());
    this->boxIs(box);
  }

  // ******************************************************************
  // ACCESSORS


  // ******************************************************************
  // METHODS

  /* reset table of alternative chars (and free memory) */
  int Glyph::reset_box_ac() {
    #ifdef INLINE_SomeName
    // If Glyph IS Box, we do this (normally Glyph has-a Box)
    return ::reset_box_ac((struct box*)this);
    #else
    return ::reset_box_ac(this->boxRef());
    #endif
  }

  int Glyph::free_box() {
    // free alternative char table
    #ifdef INLINE_SomeName
    // If Glyph IS Box, we do this (normally Glyph has-a Box)
    return ::reset_box_ac((struct box*)this); 
    #else
    return ::reset_box_ac(this->boxRef()); 
    #endif
  }

  /// Add the contents of given glyph to myself, in particular,
  /// the vectors of the glyph.
  int Glyph::merge_boxes(Glyph* glyph) {
    #ifdef INLINE_SomeName
    // If Glyph IS Box, we do this (normally Glyph has-a Box)
    return ::merge_boxes((struct box*)this,(struct box*)box2);
    #else
    return ::merge_boxes(this->boxRef(),glyph->boxRef());
    #endif
  }

  /* simplify the vectorgraph, 
   *  but what is the best way?
   *   a) melting two neighbouring vectors with nearly same direction?
   *      (nearest angle to pi)
   *   b) melting three neigbours with smallest area?
   * ToDo:
   * mode = 0 - only lossless
   * mode = 1 - reduce one vector, smallest possible loss
   * mode = 2 - remove jitter (todo, or somewhere else)
   * ToDo: include also loop around (last - first element)
   * ToDo: reduce by 10..50%
   */
  int Glyph::reduce_vectors(int mode ) {
    #ifdef INLINE_SomeName
    // If Glyph IS Box, we do this (normally Glyph has-a Box)
    return ::reduce_vectors((struct box*)this,mode);
    #else
    return ::reduce_vectors(this->boxRef(),mode);
    #endif
  }

  /* after a box is splitted into 2, where vectors are copied to both
   * vectors outside the new box are cutted and thrown away
   * later replaced by
   *  - adding two (ore more) vectors on the cutting points and
   *  - split add this points, connect only in-out...out-in,
   * ToDo: dont connect  --1---2--------3----4--
   *                        \   \------/    /    outside (inside ok?)
   *                         \-------------/
   *
   *       only connect  --1---2--------3----4--
   *                        \--/        \---/    outside
   */
  int Glyph::cut_box() {
    #ifdef INLINE_SomeName
    // If Glyph IS Box, we do this
    return ::cut_box((struct box*)this);
    #else
    return ::cut_box(this->boxRef());
    #endif
  } 

  void Glyph::printSummary(ostream& os, const string& indent, int index) const {
    os << indent << "Glyph ";
    if ( index >= 0 ) { os << right << setw(4) << index; }
    const Box* box = this->box();

    os << ": "
       << " id="   << left << setw(10) << this->id().c_str()
       << " tl="   << this->topleft().str(3)
       << " br="   << this->bottomright().str(3)
       << " sz="   << this->size().str(2)
       << " m1="   << setw(3) << box->m1
       << " m2="   << setw(3) << box->m2
       << " m3="   << setw(3) << box->m3
       << " m4="   << setw(3) << box->m4
       << " c="    << box->c
       << endl;
  }

  void Glyph::test() {
    cerr << "ERROR: Not unit test for Glyph - FIX THIS!" << endl;
  }

};
