
// C++ Dependencies
# include <iostream>
# include "Image.h"  
# include "Element.h"  // Coord
# include "Env.h"   // FUTURE FIX: only needed temporarily for 'test'

// C Dependencies (temporary)
# include "pnm.h"      // readpnm, writepgm, writeppm, writepbm
# include "pgm2asc.h"  // copybox


using namespace std;

// ******************************************************************
// CONSTRUCTORS
namespace SomeName {
  Image::Image(const string& file, bool verbose) {
    if ( verbose ) { cerr << "NOTE: Reading PNM file " << file << endl; }
    char* fn  = const_cast<char*>(file.c_str());
    pix* data = (pix*)malloc(sizeof(pix));
    this->dataIs(data);
    readpgm(fn, data, 0);
  }

  Image::Image(bool verbose) {
    if ( (JOB) && (JOB->src.fname) ) {
      char* file = JOB->src.fname;
      if ( verbose ) { cerr << "NOTE: Using JOB-based PNM file " << file << endl; }
      this->dataIs(&(JOB->src.p));
    } else {
      throw string("NoJobException: JOB variable is not properly initialized");
    }
  }

  Image::Image(const Image* src, const Coord& topleft, const Coord& bottomright, int magnification , bool verbose) {
    // FUTURE FIX: Currently relying on gocr C functions - must
    // rewrite!
    if ( verbose ) { 
      Coord size(src->data()->x,src->data()->y);
      cerr << "NOTE: Creating sub-image from " << topleft.str() 
	   << " to " << bottomright.str() 
	   << endl 
	   << "NOTE:    from within image of size " << size.str();
      if ( magnification > 1 ) {
	cerr << " (magnification " << magnification << ")";
      }
      cerr << endl;
    }

    if ( magnification < 1 ) {
      // Should we make 'magnification' unsigned?
      throw string("IllegalMagnificationException: magnification must be >= 1");

    } else if ( src->bytesPerPixel() != 1 ) {
      // Not yet handling color - does the code below work for color
      // as long as magnitude is 1, or does it need to be changed to
      // support basic copying too?
      throw string("UnsupportedFeatureException: Not currently handling the copying and/or magnification of color images");

    } else {
      // The current implementation uses a 'pixmap' data-structure
      // nested within this Image class.  Although this code does not
      // rely on the low-level gocr C functions, we maintain the
      // pix structure because other methods in this class do still rely
      // on it.  Note that we are managing to start separating the code
      // though.
      
      // Obtain the 'pixmap' structure to be initialized (from 'this')
      pix * subdata = (pix*) malloc(sizeof(pix));  // Must use malloc because gocr does - be careful about unintentionally mixing new and malloc - nightmares!
      this->dataIs(subdata);
      
      // Prepare the fields in 'subdata'.  
      int dx = bottomright.x - topleft.x + 1;
      int dy = bottomright.y - topleft.y + 1;

      // Magnification multiples both dimensions.
      subdata->x = dx * magnification;
      subdata->y = dy * magnification;

      int bytes = subdata->x*subdata->y;
      subdata->p = (unsigned char*)malloc(bytes);
      
      // Now copy from source into destination. Note that we are not
      // using the low-level copybox, nor are we removing low bits or
      // doing pixel filtering.  Such processing will be done by
      // other methods.
      int x0 = topleft.x;
      int y0 = topleft.y;
      for (int y = 0; y < dy; y++) {
	for (int ym = 0; ym < magnification; ++ym) {
	  for (int x = 0; x < dx; x++) {
	    for (int xm = 0; xm < magnification; ++xm) {
	      // DEBUG: cerr << Coord(x*magnification+xm,y*magnification+ym).str() << " from " << Coord(x+x0,y+y0).str() << endl;
	      this->set(x*magnification+xm,y*magnification+ym, src->get(x+x0,y+y0));
	    }
	  }
	}
      }
    }
  }

  Image::~Image() {
    // The temporary 'data' field is by-value, but its 'p' field
    // is dynamically allocated via 'malloc'.
    if (this->data()->p) {
      free(this->data()->p);
      this->dataRef()->p = NULL;
    }
  }
  
  // ******************************************************************
  // ACCESSORS
  
  
  // ******************************************************************
  // METHODS

  bool Image::writeGrey(const string& filebase, bool verbose) const {
    // FUTURE FIX: All this type-violation is due to reliance on
    // C functions.  They must be replaced with type-safe C++
    // methods asap!
    char* fn  = const_cast<char*>((filebase+".pgm").c_str());
    if ( verbose ) { cerr << "NOTE: Writing PGM (grey)  file " << fn << endl; }
    pix* data = const_cast<pix*>(this->data());
    bool result = writepgm(fn,data);
    return result;
  }

  bool Image::writeColor(const string& filebase, bool verbose) const {
    // FUTURE FIX: All this type-violation is due to reliance on
    // C functions.  They must be replaced with type-safe C++
    // methods asap!
    char* fn  = const_cast<char*>((filebase+".ppm").c_str());
    if ( verbose ) { cerr << "NOTE: Writing PPM (color) file " << fn << endl; }
    pix* data = const_cast<pix*>(this->data());
    bool result = writeppm(fn,data);
    return result;
  }

  bool Image::writeMono(const string& filebase, bool verbose) const {
    // FUTURE FIX: All this type-violation is due to reliance on
    // C functions.  They must be replaced with type-safe C++
    // methods asap!
    char* fn  = const_cast<char*>((filebase+".pbm").c_str());
    if ( verbose ) { cerr << "NOTE: Writing PBM (mono)  file " << fn << endl; }
    pix* data = const_cast<pix*>(this->data());
    bool result = writepbm(fn,data);
    return result;
  }

  void Image::test() {
    Env::Initialize();  // deal with problematic global-var JOB

    bool verbose = true;
    Image* im = new Image("test.pnm", verbose);

    Image* subim = new Image(im, Coord(20,20), Coord(60,60), 1, verbose);
    subim->writeGrey("tmp/outsub", verbose);

    Image* subim2 = new Image(im, Coord(20,20), Coord(60,60), 2, verbose);
    subim2->writeGrey("tmp/outsub2", verbose);

    Image* subim3 = new Image(im, Coord(20,20), Coord(60,60), 3, verbose);
    subim3->writeGrey("tmp/outsub3", verbose);

    im->writeGrey("tmp/out", verbose);
    im->writeMono("tmp/out", verbose);
    im->writeColor("tmp/out", verbose);

    delete subim3;
    delete subim2;
    delete subim;
    delete im;
  }

};

