# ifndef Line_h
# define Line_h Line_h


// Dependencies
#include "Element.h" // parent

// Declaration
namespace SomeName {

  // ###################################################################
  /// \class Line
  ///
  /// \brief A sub-region of a Page (or Region) consisting entirely
  /// of Glyphs.  
  ///
  /// My be sub-divided into Words that contain Glyphs, but may
  /// also contain only Glyphs.  Line classes are not strictly
  /// necessary during character identification, but are useful at
  /// least during output to ensure that characters on the same line
  /// in the input image are placed on the same text line in output.
  ///
  /// Whether Lines are detected first (and then analyed to detect
  /// Glyphs), or whether Glyphs are detected first (and then grouped to
  /// form Lines) is part of the partitioning process, and is
  /// under client control.
  // ###################################################################

  class Line : public Element {
  public:
    // Constructors/Destructors
    Line(Element* parent, const Coord& topleft, const Coord& bottomright);
    
    // Accessors
    virtual int type() const { return Type; }
    
    // Input/Output
    
    // Miscellaneous

    /// Allows Image instances to be converted to Line instances
    /// safely without requiring explicit down-casting.
    virtual Line*   asLine()   { return this; }
    virtual const Line*   asLine() const { return this; }

    virtual void printSummary(std::ostream& os = std::cerr, const std::string& indent = "", int index = -1) const;

    /// Unit testing method.
    ///
    /// This static method should create instances of the class (and
    /// instances of any other class necessary) and perform tests to
    /// ensure that all methods within the class are working as
    /// expected.
    static void test();

    
  protected:
    // Accessors
    
    // Methods 
    
  private: 
    // Accessors
    
    // Methods 
    
    // State
    static const int Type = 3;
  };
};

# endif // Line_h

