# include <iomanip>   // setw
# include <fstream>   // file writing

# include "Page.h"   // me
# include "Line.h"   // has-some
# include "Glyph.h"  // has-some
# include "Image.h"  // has-

# include "StrategyFactory.h"
# include "ProcessStrategy.h"
# include "SegmentStrategy.h"
# include "IdentifyStrategy.h"
# include "FormatStrategy.h"

using namespace std;

// C Dependencies (temporary)
# include "otsu.h"     // otsu, thresholding
# include "barcode.h"  // detect_barcode
extern int progress(int p1, int p2);
extern int scan_boxes( pix *p );
extern int add_line_info();
extern int sort_box_func (const void *a, const void *b);
extern int glue_broken_chars( pix *pp );
extern void measure_pitch( Job *job );
extern int find_same_chars( pix *pp);
extern int char_recognition( pix *pp, int mo);
extern int compare_unknown_with_known_chars(pix * pp, int mo);
extern int  try_to_divide_boxes( pix *pp, int mo);
extern int list_insert_spaces( pix *pp, Job *job );
extern int context_correction( Job *job );
extern int  divide_vert_glued_boxes( pix *pp, int mo);

namespace SomeName {
  // ******************************************************************
  // CONSTRUCTORS

  Page::Page(Env* env, const string& file) : Element(NULL, Coord(0,0), Coord(0,0)) {
    // The Coord(0,0) init-list args are dirty, but the fields ARE
    // properly initialized within this constructor!.

    // Every Page maintains its environment
    this->envIs(env);

    // Initialize a Page instance from the given image file.
    this->fileIs(file);

    // FUTURE FIX: For now, we store the same Job instance in Page as
    // we do in Env. However, this will be cleaned up significantly
    // after the basic C++ interface is established.
    Job* job = env->jobRef();
    this->jobIs(job);

    // Eventually, we will be able to read the file into an Image
    // instance using the normal Image constructors. However, since we
    // are relying on gocr code currently, the 'job' field needs to be
    // initialized properly for gocr code to work, and 'job' stores a
    // pixmap for the entire file. If we create an Image using the
    // public constructors, we will end up having two pixmap
    // structures for the entire page. To avoid this, we use a special
    // (temporary) private constructor in Image that shares the
    // pixmap data of job->src.p.

    // Suffix establishes format.  FUTURE FIX: Use magic number instead.
    char* fn = job->src.fname; // fn should equal file
    if ( file != fn ) {
      throw string("Exception: Mismatched filed names in Page::Page");
    }
    bool verbose = job->cfg.verbose;
    if ( file.find(".pcx") != string::npos) {
      readpcx(fn, &job->src.p, verbose);
    } else {
      readpgm(fn, &job->src.p, verbose);
    }
    // Now create the Image using the special constructor.
    Image* image = new Image(verbose);
    this->imageIs(image);

    // Set my topleft and bottomright fields
    this->topleftIs(Coord(0,0));
    this->bottomrightIs(Coord(job->src.p.x-1,job->src.p.y-1));
  }

  // Do everything to convert this page into text.
  bool Page::process() {
    return this->algorithms()->process()->execute(this);
  }

  bool Page::segment() {
    return this->algorithms()->segment()->execute(this);
  }

  bool Page::identify() {
    return this->algorithms()->identify()->execute(this);
  }

  bool Page::format() {
    return this->algorithms()->format()->execute(this);
  }

  void Page::printSummary(ostream& os, const string& indent, int index) const {
    /// For now, we simply call the default implementation in Element,
    /// which recurses into my-subparts until Glyphs are reached,
    /// which defines a more specific 'printSummary' method.
    this->Element::printSummary(os,indent,index);

    /// FUTURE FIX: Add more code here to print out page-specific
    /// information before or after printing out the containment
    /// structure.
  }

  StrategyFactory* Page::algorithms() const {
    // FUTURE FIX: Fix this const_cast - friendship or provide public
    // non-const access to factory in Env.
    return const_cast<StrategyFactory*>(this->env()->algorithms());
  }

  void Page::test() {
    static const int argc = 5;
    static const char* argv[argc] = { "exec", "-i", "test.pnm", "-o", "test.ocr" };
    Env * env = new Env(argc,argv);
    // Ugh! This is ugly code - stupid global JOB sucks! Relying on
    // job at all sucks, because so much is happening under the hood
    // and isn't obvious. Must fix!
    Job* job = JOB;
    char* fname = job->src.fname;
    cerr << "NOTE: Loading " << fname << endl;
    Page* page = env->currentPage();
    page->segment();

    page->magnifyGlyphs("tmp", 4);
  }
  
};
