# ifndef Region_h
# define Region_h Region_h


// Dependencies
#include "Element.h" // parent

// Declaration
namespace SomeName {

  // ###################################################################
  /// \class Region
  ///
  /// \brief A sub-region of a Page, usually used to represent the 
  /// graphical area for a single column in a multi-column 
  /// image.  
  ///
  /// May be sub-divided into a collection of Lines, Words, and/or
  /// Glyphs.
  ///
  /// Currently expected to be relatively unimportant, and may actually
  /// be removed if deemed unnecessary.
  // ###################################################################

  class Region : public Element {
  public:
    // Constructors/Destructors
    Region(Element* parent, const Coord& topleft, const Coord& bottomright);
    
    // Accessors
    virtual int type() const { return Type; }
    
    // Input/Output
    
    // Miscellaneous

    /// Allows Image instances to be converted to Region instances
    /// safely without requiring explicit down-casting.
    virtual Region* asRegion() { return this; }
    virtual const Region* asRegion() const { return this; }

    virtual void printSummary(std::ostream& os = std::cerr, const std::string& indent = "", int index = -1) const;

    /// Unit testing method.
    ///
    /// This static method should create instances of the class (and
    /// instances of any other class necessary) and perform tests to
    /// ensure that all methods within the class are working as
    /// expected.
    static void test();

    
  protected:
    // Accessors
    
    // Methods 
    
  private: 
    // Accessors
    
    // Methods 
    
    // State
    static const int Type = 2;
  };
};

# endif // Region_h

