# ifndef GocrAlgorithms_h
# define GocrAlgorithms_h GocrAlgorithms_h

# include "StrategyFactory.h"  // parent

namespace SomeName {

  /// \class GocrAlgorithms
  ///
  /// \brief Uses the Gocr Strategies for all algorithms
  ///
  /// Every algorithm in the SomeName framework has an equivalent in
  /// GOCR (or the algorithm is subsumed by a higher-level algorithm
  /// in GOCR). This factory returns back the Gocr Strategies for all
  /// algorithms.
  class GocrAlgorithms : public StrategyFactory {
  public:
    virtual ProcessStrategy*   createProcessStrategy()   const;

    virtual SegmentStrategy*   createSegmentStrategy()   const;
    virtual IdentifyStrategy*  createIdentifyStrategy()  const;
    virtual FormatStrategy*    createFormatStrategy()    const;
  };
}

# endif // GocrAlgorithms_h
