// C++ Dependencies
using namespace std;
# include <iostream>
# include "GocrIdentify.h"
# include "Page.h"
# include "Glyph.h"
# include "Line.h"

// C Dependencies
# include "gocr.h"     // Job
# include "otsu.h"     // otsu, thresholding
# include "barcode.h"  // detect_barcode
extern int progress(int p1, int p2);
extern int scan_boxes( pix *p );
extern int add_line_info();
extern int sort_box_func (const void *a, const void *b);
extern int glue_broken_chars( pix *pp );
extern void measure_pitch( Job *job );
extern int find_same_chars( pix *pp);
extern int char_recognition( pix *pp, int mo);
extern int compare_unknown_with_known_chars(pix * pp, int mo);
extern int  try_to_divide_boxes( pix *pp, int mo);
extern int list_insert_spaces( pix *pp, Job *job );
extern int context_correction( Job *job );
extern int  divide_vert_glued_boxes( pix *pp, int mo);
// FUTURE FIX: Most of the above are NOT needed in this class!!

namespace SomeName {
  GocrIdentify::GocrIdentify() {
    // Default Constructor
  }

  bool GocrIdentify::execute(Page* page) {
    //DEBUG: cerr << "**** HERE in GocrIdentify::identify" << endl;

    // Perform analysis
    Job* job = const_cast<Job*>(page->job()); // FUTURE FIX: We could add this method as a friend of Page so we don't have to const_cast (use jobRef() instead)

    // Set up some local vars
    int  mode    = job->cfg.mode;
    pix* pp      = &(job->src.p);
    bool result  = true;

    // ******************************************************
    // Perform the actual character recognition.
    //
    // NOTE: Instead of calling the gocr 'char_recognition' method, we
    // provide a rewrite here that uses our data-structures instead of
    // the cumbersome gocr boxlist. Comparing this code with
    // char_recognition in pgm2asc.c may be useful for individuals
    // migrating other parts of the gocr code base to C++.

# ifdef USE_RAW_GOCR
    for_each_data(&(JOB->res.boxlist)) {
      box2 = (Box *)list_get_current(&(JOB->res.boxlist));
      x0=box2->x0;x1=box2->x1;
      y0=box2->y0;y1=box2->y1;	// box
      cc=box2->c;
      if (cc==PICTURE) continue;
      
      if ((mo&256)==0) { /* this case should be default (main engine) */
	if(cc==UNKNOWN || box2->num_ac==0 || box2->wac[0]<JOB->cfg.certainty)
	  cc=whatletter(box2,cs   ,0);
      }
      
      if(mo&2) 
	if(cc==UNKNOWN || box2->num_ac==0 || box2->wac[0]<JOB->cfg.certainty)
	  cc=ocr_db(box2);
      
      
      // box2->c=cc; bad idea (May03 removed)
      // set(box2,cc,95); ToDo: is that better? 
      
      if(cc==UNKNOWN) 
	i++;
      ii++;
      
      if(JOB->cfg.verbose&8) { 
	fprintf(stderr,"\n# code= %04lx %c",(long)cc,(char)((cc<255)?cc:'_')); 
	out_b(box2,pp,x0,y0,x1-x0+1,y1-y0+1,cs);
      }
      progress(-1,ii*100/ni);
      
    } end_for_each(&(JOB->res.boxlist));
# else 
    char_recognition( pp, job->cfg.mode);
    // NOTE: Not bothering to obtain UNKNOWN/PICTURE counts.

# endif // not USE_RAW_GOCR

    progress(60,0); /* progress is only estimated */

    // ******************************************************    
    if ( adjust_text_lines( pp, mode ) ) {
      //DEBUG: cerr << "NOTE: Sorting lines and recalling char_recognition!" << endl;
      /* correct using chars */
      /* may be, characters/pictures have changed line number */
      list_sort(&(job->res.boxlist), sort_box_func);
      // 2nd recognition call if lines are adjusted
      char_recognition( pp, job->cfg.mode);
    }

    // ******************************************************
    // write out20.pgm - mark lines + boxes
    if(job->cfg.verbose&32) debug_img("out20",job,1+4+8);

    // ******************************************************
    compare_unknown_with_known_chars( pp, job->cfg.mode);
    progress(70,0); /* progress is only estimated */

    // ******************************************************
    try_to_divide_boxes( pp, job->cfg.mode);
    progress(80,0); /* progress is only estimated */

    // ******************************************************
    // list output - for debugging
    if( job->cfg.verbose&6 ) output_list(job);

    // ******************************************************
    list_insert_spaces( pp , job );

    // ******************************************************
    // proof difficult chars Il1 by context view
    if(!(job->cfg.mode&32)) context_correction( job );

    return result;
  }
};

