# ifndef ProcessStrategy_h
# define ProcessStrategy_h ProcessStrategy_h

# include "Strategy.h" // parent

namespace SomeName {
  class Page;

  /// \class ProcessStrategy
  ///
  /// \brief The abstract superclass of a hierarchy of classes
  /// implementing the highest-level 'process' algorithm.
  ///
  /// Other strategy class hierarchies assume that processing consists
  /// of partitioning, identifying, and formatting. However, a
  /// creative individual may come up with an overall approach to OCR
  /// that does not follow this very general model. This Strategy
  /// allows for completely control over the entire OCR 'process'
  /// algorithm.
  class ProcessStrategy : public Strategy {
  public:
    ProcessStrategy();
    virtual bool execute(Page* page) = 0;
  };
}

# endif // ProcessStrategy_h

