# include <iostream>
# include "StrategyFactory.h"  

namespace SomeName {

  StrategyFactory::StrategyFactory() {
    // We CANNOT call 'init' within this constructor because
    // pure-virtual methods are not yet fully "available" within
    // constructors. After creation, we must call 'init'. This is NOT
    // a desirable feature and violates the mandate that all classes
    // be fully initialized after the constructor has executed.
    //
    // FUTURE FIX: Can this be resolved, so that the StrategyFactory
    // class IS fully initialized when the constructor returns?? One
    // "solution" would be to call 'init' within the child
    // constructors. However, that requires clients who are defining
    // StrategyFactory subclasses to remember to do so (error prone).
    //
    // Currently, the StrategyFactory subclass instance is created in
    // exactly one place (the Env::Env constructor). Within that
    // method, we ensure that 'init' is called immediately after the
    // constructor has returned. The danger comes if we create a
    // StrategyFactory someplace else, and do not realize that 'init'
    // must be callled afterwards.
  }

  void StrategyFactory::init() {
    this->processIs( this->createProcessStrategy() );
    this->segmentIs( this->createSegmentStrategy() );
    this->identifyIs( this->createIdentifyStrategy() );
    this->formatIs( this->createFormatStrategy() );
  }

};
