using namespace std;
# include <iostream>
# include <string>

// Unit testing support currently requires that we include all
// the class headers.  When we switch to a dlsym mechanism this
// won't be necessary.
# include "Env.h"
# include "Glyph.h"
# include "Image.h"
# include "Line.h"
# include "Page.h"
# include "Region.h"
# include "Root.h"
# include "Word.h"
# include "Element.h"

using namespace SomeName;

int
main(int argc, const char **argv) {

  try {
    // First, check for a special case.  If the first arg is -T,
    // it is a request to invoke a unit-testing method instead of
    // performing normal execution.
    
    if ( string(argv[1]) == "-T" ) {
      // FUTURE FIX: Use dlsym to obtain the appropriate method - much more flexible!
      if ( argc > 2 ) {
	string test = argv[2];
	
	if ( test == "Image" ) {
	  Image::test();
	} else if ( test == "Env" ) {
	  Env::test();
	  //} else if ( test == "Element" ) {
	  // Element::test();
	} else if ( test == "Page" ) {
	  Page::test();
	} else if ( test == "Region" ) {
	  Region::test();
	} else if ( test == "Line" ) {
	  Line::test();
	} else if ( test == "Word" ) {
	  Word::test();
	} else if ( test == "Glyph" ) {
	  Glyph::test();
	} else {
	  cerr << "WARN: Unit testing of class " << test << " not yet supported" << endl;
	}
      } else {
	cerr << "ERROR: -T requires an argument specifying the class to test" << endl;
      }
      
    } else {
      
      // We create an Env instance.
      Env* env = new Env(argc, argv);
      
      // Obtain the current page (identified by command-line args)
      Page* page = env->currentPage();
      
      // Process the page
      page->process();
      
      // Now wrapup.
      env->wrapup();
    }
      
  } catch(std::string& e) {
    // FUTURE FIX: Will start throwing more intelligent exceptions later
    cerr << endl << e << endl;
    
  }

  return(0);
}
